/*
 * =============================================================================
 * 
 *   Copyright (c) 2011, The THYMELEAF team (http://www.thymeleaf.org)
 * 
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * 
 * =============================================================================
 */
package org.thymeleaf.spring3.util;

import java.util.Arrays;
import java.util.List;

import org.springframework.util.StringUtils;
import org.springframework.web.servlet.support.BindStatus;
import org.springframework.web.servlet.support.RequestContext;
import org.thymeleaf.Arguments;
import org.thymeleaf.exceptions.AttrProcessorException;
import org.thymeleaf.exceptions.TemplateProcessingException;
import org.thymeleaf.spring3.naming.SpringContextVariableNames;
import org.thymeleaf.standard.processor.value.IStandardValueProcessor;
import org.thymeleaf.standard.syntax.StandardSyntax;
import org.thymeleaf.standard.syntax.StandardSyntax.Value;
import org.thymeleaf.standard.syntax.StandardSyntax.VarValue;
import org.thymeleaf.standard.syntax.StandardSyntax.VarValue.VarExpressionType;
import org.thymeleaf.templateresolver.TemplateResolution;



/**
 * 
 * @author Daniel Fern&aacute;ndez
 * 
 * @since 1.0
 *
 */
public class FieldUtils {

    public static final String ALL_FIELDS = "*";
    public static final String ALL_FIELDS_FIELD_EXPRESSION = "*{*}";
    

    
    

    
    
    public static boolean hasErrors(
            final Arguments arguments, final TemplateResolution templateResolution,
            final IStandardValueProcessor valueProcessor, final String field) {

        final BindStatus bindStatus = 
            FieldUtils.getBindStatus(arguments, templateResolution, valueProcessor, convertToFieldExpression(field), true);
        
        return bindStatus.isError();
        
    }
    
    

    public static List<String> errors(
            final Arguments arguments, final TemplateResolution templateResolution,
            final IStandardValueProcessor valueProcessor, final String field) {

        final BindStatus bindStatus = 
            FieldUtils.getBindStatus(arguments, templateResolution, valueProcessor, convertToFieldExpression(field), true);
        
        final String[] errorCodes = bindStatus.getErrorMessages();
        return Arrays.asList(errorCodes);
        
    }


    public static String idFromName(final String fieldName) {
        return StringUtils.deleteAny(fieldName, "[]");
    }
    
    
    
    private static String convertToFieldExpression(final String field) {
        return 
            Character.toString(StandardSyntax.SELECTION_VARIABLE_SELECTOR_CHAR) +
            Character.toString(StandardSyntax.EXPRESSION_START_CHAR) + 
            field + 
            Character.toString(StandardSyntax.EXPRESSION_END_CHAR);
    }
    
    

    
    
    
    public static BindStatus getBindStatus(
            final Arguments arguments, final TemplateResolution templateResolution,
            final IStandardValueProcessor valueProcessor, final String fieldExpression, final boolean allowAllFields) {
        
        final RequestContext requestContext =
            (RequestContext) arguments.getContext().getVariables().get(SpringContextVariableNames.SPRING_REQUEST_CONTEXT);
        if (requestContext == null) {
            throw new AttrProcessorException("A request context has not been created");
        }

        if (allowAllFields && ALL_FIELDS_FIELD_EXPRESSION.equals(fieldExpression)) {
            
            final String completeExpression = 
                FieldUtils.validateAndGetValueExpressionForAllFields(arguments, templateResolution);
            
            return new BindStatus(requestContext, completeExpression, false);
            
        }
        
        final Value value = 
            StandardSyntax.parseValue(fieldExpression, valueProcessor, arguments, templateResolution);
        if (value == null) {
            throw new AttrProcessorException(
                    "Could not parse attribute value: \"" + fieldExpression + "\"");
        }
        
        final String completeExpression = 
            FieldUtils.validateAndGetValueExpressionForField(arguments, templateResolution, valueProcessor, value);
        
        return new BindStatus(requestContext, completeExpression, false);
        
    }
    

    
    
    
    private static String validateAndGetValueExpressionForField(
            final Arguments arguments, final TemplateResolution templateResolution, 
            final IStandardValueProcessor valueProcessor, final Value value) {

        /*
         * Only asterisk syntax (selection variable expressions) are allowed here.
         */
        
        if (value instanceof VarValue) {

            final VarValue varValue = (VarValue) value;
            if (varValue.getExpressionType().equals(VarExpressionType.SELECTION_ROOT)) {
                final VarValue formCommandValue = 
                    (VarValue) arguments.getLocalVariable(SpringContextVariableNames.SPRING_FORM_COMMAND_VALUE);
                if (formCommandValue == null) {
                    throw new TemplateProcessingException(
                            "Cannot process field expression " + value + " as no form model object has " +
                            "been established in the \"form\" tag");
                }
                final String formCommandExpression = formCommandValue.getExpression();
                return formCommandExpression + "." + varValue.getExpression();
            }
            
        }
        
        throw new TemplateProcessingException(
                "Expression \"" + value + "\" is not valid: only selection variable expressions " +
                "*{...} are allowed in field specifications");
        
    }
    
    
    
    private static String validateAndGetValueExpressionForAllFields(
            final Arguments arguments, final TemplateResolution templateResolution) {

        final VarValue formCommandValue = 
            (VarValue) arguments.getLocalVariable(SpringContextVariableNames.SPRING_FORM_COMMAND_VALUE);
        if (formCommandValue == null) {
            throw new TemplateProcessingException(
                    "Cannot process expression for all fields \"" + ALL_FIELDS + "\" as no form model object has " +
                    "been established in the \"form\" tag");
        }
        final String formCommandExpression = formCommandValue.getExpression();
        return formCommandExpression + "." + ALL_FIELDS;
        
    }
    
    

	private FieldUtils() {
	    super();
	}

}
